/// @copyright
/// Copyright (C) 2020 Assured Information Security, Inc.
///
/// @copyright
/// Permission is hereby granted, free of charge, to any person obtaining a copy
/// of this software and associated documentation files (the "Software"), to deal
/// in the Software without restriction, including without limitation the rights
/// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
/// copies of the Software, and to permit persons to whom the Software is
/// furnished to do so, subject to the following conditions:
///
/// @copyright
/// The above copyright notice and this permission notice shall be included in
/// all copies or substantial portions of the Software.
///
/// @copyright
/// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
/// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
/// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
/// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
/// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
/// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
/// SOFTWARE.
///
/// @file is_same.hpp
///

#ifndef BSL_IS_SAME_HPP
#define BSL_IS_SAME_HPP

#include "bsl/false_type.hpp"
#include "bsl/integral_constant.hpp"    // IWYU pragma: export
#include "bsl/true_type.hpp"

namespace bsl
{
    /// @class bsl::is_same
    ///
    /// <!-- description -->
    ///   @brief If the provided types are the same, provides the member
    ///     constant value equal to true. Otherwise the member constant value
    ///     is false.
    ///   @include example_is_same_overview.hpp
    ///
    /// <!-- template parameters -->
    ///   @tparam T the type that should be the same as U
    ///   @tparam U the type that should be the same as T
    ///
    template<typename T, typename U>
    class is_same final : public false_type
    {};

    /// @cond doxygen off

    template<typename T>
    class is_same<T, T> final : public true_type
    {};

    /// @endcond doxygen on
}

#endif
